@@ -8,19 +8,19 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-use crate::arm::{self, F32x4};
+use crate::arm::{self, F32x4, Simd};
 
 macro_rules! simd_shuffle4 {
     ($x:expr, $y:expr, <$(const $imm:ident : $ty:ty),+> $idx:expr $(,)?) => {{
         struct ConstParam<$(const $imm: $ty),+>;
         impl<$(const $imm: $ty),+> ConstParam<$($imm),+> {
-            const IDX: [u32; 4] = $idx;
+            const IDX: Simd<u32, 4> = Simd($idx);
         }
 
         arm::simd_shuffle($x, $y, ConstParam::<$($imm),+>::IDX)
     }};
     ($x:expr, $y:expr, $idx:expr $(,)?) => {{
-        const IDX: [u32; 4] = $idx;
+        const IDX: Simd<u32, 4> = Simd($idx);
         arm::simd_shuffle($x, $y, IDX)
     }};
 }
